module BaseEnumz
  attr_reader :data_elements, :options

  # list of ruby module method names that
  # cannot be overridden
  # i.e. overridding enum.class is too destructive
  RESERVED_METHOD_NAMES = [:class, :object_id, :method_missing,
                           :instance_eval, :send, :should_not, :nil?,
                           :const_get, :const_missing, :inspect, :to_s]

  # return list of enum names
  def names
    @names ||= data_elements.keys
  end

  # return list of enum values
  def values
    @values ||= data_elements.values
  end

  # return the corresponding enum name for a given value
  def [](value)
    @inverted_data_elements ||= data_elements.invert
    @inverted_data_elements[value]
  end

  # create a method for each enum name
  def method_missing(meth, *args, &block)
    if data_elements[meth.to_s.to_sym]
      self.instance_eval <<-EVAL
        def #{meth}
          data_elements[:#{meth}]
        end
      EVAL

      __send__(meth, *args, &block)
    else
      super
    end
  end

end # module BaseEnumz
